
import httplib
import simplejson as json
import logging
import mimetypes
import socket
from urllib import urlencode

import httplib2
import socks
httplib2.socks = socks

USER_AGENT = 'auto_update'
COOKIE = ''
PROXY_HOST = ''
PROXY_PORT = 80
PROXY_AUTH_BASIC = ''

timeout = None

__rc = 0
logger = logging.getLogger('HTTP')

class fpwrap(object):
    def __init__(self, fp):
        self.fp = fp
        
    def read(self, amt):
        return self.fp.read(amt)
        
    def getattr(self, k):
        return getattr(self.fp, k)

class HTTPResponseWithProgress(httplib.HTTPResponse):
    progress_bufsize = 4096
    progress_callback = None
    
    def __init__(self, *args, **kwargs):
        httplib.HTTPResponse.__init__(self, *args, **kwargs)
        self._progress_enable = False
        self._progress_callback = self.progress_callback
        self._progress_bufsize = self.progress_bufsize
        self._progress = 0
        
    def begin(self):
        httplib.HTTPResponse.begin(self)
        if self.length is not None and self.progress_callback is not None:
            self.fp = fpwrap(self.fp)
            self.__fpread = self.fp.read
            self.fp.read = self.__progress_read
    
    def read(self, amt=None):
        if self.length is not None and amt is None:
            self.initial_length = self.length
            amt = self.progress_bufsize
            data = []
            while self.length > 0:
                data.append(httplib.HTTPResponse.read(self, amt))
            return ''.join(data)
        else:
            return httplib.HTTPResponse.read(self, amt)
        
    def __progress_read(self, amt=None):
        if amt is None:
            r = self.__fpread()
        else:
            r = self.__fpread(amt)
        self._progress += len(r)
        self._progress_callback(self._progress, self.initial_length)
        return r

class HTTPConnectionWithTimeoutAndProgress(httplib2.HTTPConnectionWithTimeout):
    progress_bufsize = 4096
    progress_callback = None
    response_class = HTTPResponseWithProgress
    
    def __init__(self, host, port=None, strict=None, timeout=None, proxy_info=None):
        httplib2.HTTPConnectionWithTimeout.__init__(self, host, port, strict, timeout, proxy_info)
        self._progress_enable = False
        self._progress_callback = self.progress_callback
        self._progress_bufsize = self.progress_bufsize
    def endheaders(self):
        httplib2.HTTPConnectionWithTimeout.endheaders(self)
        self._progress_enable = True
    def send(self, str):
        import socket
        """Send `str' to the server."""
        if self.sock is None:
            if self.auto_open:
                self.connect()
            else:
                raise NotConnected()

        # send the data to the server. if we get a broken pipe, then close
        # the socket. we want to reconnect when somebody tries to send again.
        #
        # NOTE: we DO propagate the error, though, because we cannot simply
        #       ignore the error... the caller will know if they can retry.
        if self.debuglevel > 0:
            print "send:", repr(str)
        try:
            i = 0
            while i < len(str):
                c = self.sock.send(str[i:i+self._progress_bufsize])
                i = i + c
                if self._progress_enable and self._progress_callback is not None:
                    self._progress_callback(i, len(str))
        except socket.error, v:
            if v[0] == 32:      # Broken pipe
                self.close()
            raise


def request(url, method='GET', body=None, headers={}, progress_callback=None, progress_bufsize=4096):
    global __rc, COOKIE, USER_AGENT, PROXY_HOST, PROXY_PORT, PROXY_AUTH_BASIC, timeout, logger
    __rc = __rc + 1

    logger.info('[%d] %s %s' % (__rc, method, url))

    if body is not None:
        if type(body) is dict:
            body = urlencode(body)
            headers['Content-type'] = 'application/x-www-form-urlencoded'
        elif type(body) is not str:
            raise ValueError("body data must be dict or string")

    headers['User-agent'] = USER_AGENT
    if COOKIE != '':
        headers['cookie'] = COOKIE

    logger.debug('[%d] Headers: %s' % (__rc, `headers`))
    if type(body) == str:
        logger.debug('[%d] Body Length: %d' % (__rc, len(body)))
        if len(body) < 256:
            logger.debug('[%d] Body Text:\n%s' % (__rc, body))

    possible_connection_errors = [httplib2.HttpLib2Error]
    if PROXY_HOST:
        possible_connection_errors.append(socks.ProxyError)
        proxy = httplib2.ProxyInfo(proxy_type=socks.PROXY_TYPE_HTTP, proxy_host=PROXY_HOST, proxy_port=PROXY_PORT)
        if PROXY_AUTH_BASIC:
            headers['Proxy-Authorization'] = 'Basic ' + PROXY_AUTH_BASIC
    else:
        proxy = None
    possible_connection_errors = tuple(possible_connection_errors)
    h = httplib2.Http(timeout=timeout, proxy_info=proxy)
    HTTPConnectionWithTimeoutAndProgress.progress_callback = progress_callback
    HTTPConnectionWithTimeoutAndProgress.progress_bufsize = progress_bufsize
    HTTPResponseWithProgress.progress_callback = progress_callback
    HTTPResponseWithProgress.progress_bufsize = progress_bufsize
    try:
        resp, content = h.request(url, method, body, headers, connection_type=HTTPConnectionWithTimeoutAndProgress)
    except socket.error, e:
        msg = e[1]
        if e[0] in [60, 61]:
            msg += ' - please check your connection and proxy settings'
        resp = {'status': e[0], 'content-type': 'text/plain', 'reason': msg}
        content = msg
    except possible_connection_errors, e:
        resp = {'status': 61, 'content-type': 'text/plain', 'reason': str(e)}
        content = str(e)
    e = {'headers':resp}

    if resp['content-type'] in ['application/json', 'application/javascript', 'text/javascript']:
        e['body'] = content
        try:
            r = json.loads(content)
            for k,v in r.iteritems():
                e[str(k)] = v
        except:
            e['code'] = resp['status']
            e['status'] = 'fail'
            e['reason'] = 'Bad server response'

        else:
            if not 'status' in e: e['status'] = 'ok'
            if not 'code' in e: e['code'] = resp['status']
            e['cookie'] = resp.get('set-cookie', '')
            e['reason'] = e.get('reason', '')
            if e['cookie'] != '': COOKIE = e['cookie']

    elif resp['status'] in ('200', '301'):
        e['code'] = resp['status']
        e['status'] = 'ok'
        e['reason'] = ''
        e['body'] = content

    elif 'text/plain' in resp['content-type']:
        if len(content) > 0 and len(content) < 120:
            reason = content
        else:
            reason = httplib.responses[int(resp['status'])]
        e['code'] = resp['status']
        e['status'] = 'fail'
        e['reason'] = reason
        e['body'] = content

    else:
        e['code'] = resp['status']
        e['status'] = 'fail'
        e['reason'] = httplib.responses[int(resp['status'])]
        # e['reason'] = 'Unknown response type'
        e['body'] = content
    
    e['code'] = int(e['code'])
    
    logger.info('[%d] %s %s' % (__rc, str(resp.get('status', '?')), e.get('reason', '')))

    logger.debug('%s' % (`e`))

    return e

