from Config import Config

from lib import logger
from ui import dialog
from ui.controls import sizers, EditableListBox, CheckListBox, TextCtrl, Label

class SaveDialog(dialog.DialogBase):
    def __init__(self, parent, windows=[]):
        self.windows = windows
        dialog.DialogBase.__init__(self, parent, 'Save Workspace')
        
    def CreatePanel(self):
        self.name = TextCtrl(self, 'Workspace %d' % (len(Config.get('workspaces', {})),))
        self.list = CheckListBox(self, choices=[w.get_workspace_description() for w in self.windows])
        for i in range(0, len(self.windows)):
            self.list.Check(i)
        
        vsizer = sizers.VBox()
        vsizer.AddWidget(Label(self, 'Workspace Name:'), 0, padding=4)
        vsizer.AddWidget(self.name, 0, padding=4)
        vsizer.AddWidget(self.list, 1)
        self.SAVE = dialog.OK
        self.CANCEL = dialog.CANCEL
        return vsizer
    
    def GetButtons(self):
        return (
            {'id': dialog.CANCEL, 'label': 'Cancel', 'onclick': self.OnClose},
            {'id': dialog.OK, 'label': 'Save', 'onclick': self.OnSave}
        )
        
    def OnSave(self, e):
        res = self.CANCEL
        name = self.name.GetValue()
        if name != '':
            msg = logger.begin('Saving Workspace', 'workspaces.save', name, loud=True)
            try:
                workspace = {'windows':[]}
                i = 0;
                for v in self.windows:
                    if self.list.IsChecked(i):
                        workspace['windows'].append(v.get_workspace_data())
                if len(workspace['windows']) > 0:
                    Config.workspaces[name] = workspace
                    res = self.SAVE
            except Exception, e:
                msg.error(e)
            else:
                msg.complete()
        self.EndModal(res)


class WorkspaceList(EditableListBox):
    
    def CreateButtons(self, parent):
        self.entry_label = 'Enter a new name for this workspace'
        self.open = self.CreateButton(parent, 'Open', self.OnOpen)
        self.edit = self.CreateButton(parent, 'Rename', self.OnEdit)
        self.remove = self.CreateButton(parent, 'Remove', self.OnRemove)
        self.Parent.OPEN = self.open.GetId()
        return [self.open, self.edit, self.remove]
        
    def UpdateState(self, have_selection):
        EditableListBox.UpdateState(self, have_selection)
        self.open.Enable(have_selection)
        
    def OnOpen(self, e):
        self.Parent.EndModal(self.Parent.OPEN)
    
    def RemoveSelected(self):
        del Config.workspaces[self.GetSelection()]
        EditableListBox.RemoveSelected(self)

    def RenameSelected(self, text):
        sel = self.GetSelection()
        Config.workspaces[text] = Config.workspaces[sel]
        del Config.workspaces[sel]
        EditableListBox.RenameSelected(self, text)
        
    def OnDoubleClick(self, e):
        self.OnOpen(e)

class Dialog(dialog.DialogBase):
    def __init__(self, parent):
        dialog.DialogBase.__init__(self, parent, 'Workspaces')
        
    def CreatePanel(self):
        self.list = WorkspaceList(self, choices=Config.get('workspaces', {}).keys())
        self.list.SetMinSize((300, 200))
        return self.list
        # vsizer = sizers.VBox()
        # vsizer.AddWidget(self.list, 1, padding=4)
        # 
        # self.SetSizer(vsizer)
        
    def GetButtons(self):
        return {}
        
    def GetSelection(self):
        return self.list.GetSelection()
        