
import wx

from Config import Config
import Window

class Dialog(wx.Dialog):
    ID_DIFF = wx.NewId()
    def __init__(self, parent, location, title, items):
        self.items = items
        
        wx.Dialog.__init__(self, parent, -1, title, style=wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        label = wx.StaticText(self, -1, "Please enter a comment")
        sizer.Add(label, 0, wx.ALIGN_LEFT|wx.ALL, 4)
        self.tb = wx.TextCtrl(self, -1, style=wx.TE_MULTILINE)
        sizer.Add(self.tb, 1, wx.GROW|wx.ALIGN_CENTER|wx.ALL, 4)
        
        label = wx.StaticText(self, -1, "Checking in items:")
        sizer.Add(label, 0, wx.ALIGN_LEFT|wx.ALL, 4)
        self.list = wx.CheckListBox(self, -1, choices=([location.relativePath(i.path()) for i in items]))
        for i in range(0, len(items)):
            self.list.Check(i)
        sizer.Add(self.list, 1, wx.GROW|wx.ALIGN_CENTER|wx.ALL, 4)
        self.cbRecursive = wx.CheckBox(self, -1, 'Recursive')
        sizer.Add(self.cbRecursive, 0, wx.EXPAND|wx.ALL, 4)
        sizer.Add(self.CreateButtons(), 0, wx.CENTRE|wx.ALL, 8)
        
        self.SetSizer(sizer)
        sizer.Fit(self)
        settings = Config.RestoreWindow(self, 'CheckinWindow.Dialog')
        if settings:
            self.cbRecursive.SetValue(settings.get('recursive', True))
        else:
            self.CenterOnParent()
            self.SetSize((500, 350))
        wx.EVT_CLOSE(self, self.OnClose)
    
    recursive = property(lambda self:self.cbRecursive.GetValue())
    comment = property(lambda self:self.tb.GetValue())
    
    def GetItems(self):
        return [self.items[i] for i in range(self.list.GetCount()) if self.list.IsChecked(i)]
        
    def CreateButtons(self):
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(wx.Button(self, wx.ID_OK, 'OK'), 0, wx.CENTRE|wx.ALL, 4)
        sizer.Add(wx.Button(self, wx.ID_CANCEL, 'Cancel'), 0, wx.CENTRE|wx.ALL, 4)
        sizer.Add(wx.Button(self, self.ID_DIFF, 'Diff'), 0, wx.CENTRE|wx.ALL, 4)
        wx.EVT_BUTTON(self, self.ID_DIFF, self.OnDiff)
        wx.EVT_BUTTON(self, wx.ID_OK, self.OnClose)
        return sizer
    def OnDiff(self, e):
        wx.PostEvent(self.GetParent(), Window.RunToolEvent(tool='svn.diff', items=self.GetItems()))
    def OnClose(self, event):
        self.SaveConfig()
        event.Skip()
    def SaveConfig(self):
        Config.SaveWindow(self, 'CheckinWindow.Dialog', {'recursive':self.cbRecursive.GetValue()})
        Config.save()
