import wx
import wx.stc as stc

from Config import Config


class DiffCtrl(stc.StyledTextCtrl):
    def __init__(self, parent, pos=wx.DefaultPosition, size=wx.DefaultSize, style=0):
        stc.StyledTextCtrl.__init__(self, parent, wx.NewId(), pos, size, style)
        self.SetLexer(stc.STC_LEX_DIFF)
        self.SetMargins(0,0)
        self.SetViewWhiteSpace(False)
        # self.SetUseAntiAliasing(True)
        font = Config.getFont('fixed')
        self.StyleSetSpec(stc.STC_STYLE_DEFAULT, "face:%s,size:%d,bold,fore:#aaaaaa" % (font.GetFaceName(), font.GetPointSize()))
        self.StyleClearAll()
        self.StyleSetSpec(stc.STC_DIFF_COMMENT, "fore:#666666")
        self.StyleSetSpec(stc.STC_DIFF_COMMAND, "fore:#660066")
        self.StyleSetSpec(stc.STC_DIFF_HEADER, "fore:#666666")
        self.StyleSetSpec(stc.STC_DIFF_POSITION, "fore:#000099")
        self.StyleSetSpec(stc.STC_DIFF_DELETED, "fore:#990000")
        self.StyleSetSpec(stc.STC_DIFF_ADDED, "fore:#009900")
    def write(self, text):
        self.AppendText(text)

class CodeDialog(wx.Dialog):
    def __init__(self, parent, format, text, title="Code", pos=wx.DefaultPosition, size=wx.DefaultSize, saveid=None):
        if saveid is None: saveid = format
        self.saveid = saveid
        self.format = format
        wx.Dialog.__init__(self, parent, wx.NewId(), title, pos=pos, size=size, style=wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)
        if not Config.RestoreWindow(self, 'CodeDialog' + self.saveid):
            self.CenterOnParent()
        Klass = globals()[format.capitalize() + 'Ctrl']
        self.stc = Klass(self)
        self.stc.SetText(text)
        self.stc.Colourise(0, -1)

        vsizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(vsizer)
        vsizer.Add(self.stc, 1, wx.EXPAND|wx.ALL, 1)
        vsizer.Add(self.CreateButtons(), 0, wx.CENTRE|wx.ALL, 8)
        
        wx.EVT_BUTTON(self, wx.ID_OK, self.OnOK)
        wx.EVT_CLOSE(self, self.OnClose)
        wx.EVT_KEY_UP(self.stc, self.OnKeyUp)
        
    def CreateButtons(self):
        return self.CreateSeparatedButtonSizer(wx.OK)
        
    def OnKeyUp(self, event):
        if event.GetKeyCode() == wx.WXK_ESCAPE:
            self.Close()
        event.Skip()
    def OnOK(self, event):
        self.Close()
    def OnClose(self, event):
        Config.SaveWindow(self, 'CodeDialog' + self.saveid)
        Config.save()
        event.Skip()

class DiffDialog(CodeDialog):
    ID_CI = wx.NewId()
    ID_REVERT = wx.NewId()
    ID_UP = wx.NewId()
    def __init__(self, parent, text, items=[], title="Code", pos=wx.DefaultPosition, size=wx.DefaultSize, saveid=None):
        self.items = items
        CodeDialog.__init__(self, parent, 'diff', text, title, pos, size, saveid)

    def CreateButtons(self):
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(wx.Button(self, wx.ID_OK, 'Close'), 0, wx.CENTRE|wx.ALL, 4)
        if len(self.items):
            if len([i for i in self.items if i.isModified()]) > 0:
                sizer.Add(wx.Button(self, self.ID_CI, 'Check In'), 0, wx.CENTRE|wx.ALL, 4)
                wx.EVT_BUTTON(self, self.ID_CI, self.OnCheckIn)
                sizer.Add(wx.Button(self, self.ID_REVERT, 'Revert'), 0, wx.CENTRE|wx.ALL, 4)
                wx.EVT_BUTTON(self, self.ID_REVERT, self.OnRevert)
            if len([i for i in self.items if i.isOutOfDate()]) > 0:
                sizer.Add(wx.Button(self, self.ID_UP, 'Update'), 0, wx.CENTRE|wx.ALL, 4)
                wx.EVT_BUTTON(self, self.ID_UP, self.OnUpdate)
        return sizer
    def OnCheckIn(self, event):
        self.Close()
        self.SetReturnCode(self.ID_CI)
    def OnRevert(self, event):
        self.Close()
        self.SetReturnCode(self.ID_REVERT)
    def OnUpdate(self, event):
        self.Close()
        self.SetReturnCode(self.ID_UP)

class DiffWindow(wx.Frame):
    ID_CLOSE = wx.NewId()
    def __init__(self, parent, text):
        wx.Frame.__init__(self, parent, -1, 'Differences')
        self.settings = Config.RestoreWindow(self, self.GetConfigId(), self.GetDefaultConfig())
        if len(self.settings) == 0:
            self.SetSize((740, 500))
            self.CenterOnParent()
        self.SetBackgroundStyle(wx.BG_STYLE_COLOUR)
        self.SetBackgroundColour(wx.Colour(232,232,232))

        self.stc = DiffCtrl(self)
        self.stc.SetText(text)
        self.stc.Colourise(0, -1)
        
        vsizer = wx.BoxSizer(wx.VERTICAL)
        vsizer.Add(self.stc, 1, wx.EXPAND|wx.ALL)
        self.SetSizer(vsizer)

        wx.EVT_CLOSE(self, self.OnClose)

        menuBar = wx.MenuBar()
        menu1 = wx.Menu()
        menu1.Append(self.ID_CLOSE, "&Close\tCtrl-W", "Close this window")
        menuBar.Append(menu1, '&File')
        self.SetMenuBar(menuBar)
        wx.EVT_MENU(self, self.ID_CLOSE, self.OnMenu)
        wx.EVT_KEY_UP(self.stc, self.OnKeyUp)

    def OnKeyUp(self, event):
        if event.GetKeyCode() == wx.WXK_ESCAPE:
            self.Close()
        event.Skip()

    def OnMenu(self, e):
        id = e.GetId()
        if id == self.ID_CLOSE:
            self.Close()
            self.Destroy()

    def GetConfigId(self):
        return self.__module__ + '.' + self.__class__.__name__

    def GetDefaultConfig(self):
        return {}

    def GetCurrentConfig(self):
        return {}

    def SaveState(self):
        try:
            Config.SaveWindow(self, self.GetConfigId(), self.GetCurrentConfig())
            Config.save()
        except Exception, e:
            print str(e)

    def OnClose(self, e):
        self.SaveState()
        e.Skip()

