from __future__ import with_statement
import os, posixpath, stat

import wx
import wx.lib.platebtn as platebtn

import AppInfo
from Config import Config
import Connection
import Images
import Location
import Model
import Plugins
from . import controls, Events, UrlPanel, Window
from lib import logger

# global cl
# def showfocus():
#     print wx.Window.FindFocus()
#     cl.Restart()
# cl = wx.CallLater(3000, showfocus)

platebtn.PlateButton.AcceptsFocusFromKeyboard = lambda self:False

class Tool(Window.Tool):
    def IsAble(self):
        return self.parent.IsValid()
    def _location(self):
        if self.parent:
            return self.parent.location
        return None
    location = property(_location)
    PostConnectionJob = property(lambda self: self.parent.PostConnectionJob)


EVT_RUN_ACTION, RunActionEvent = Events.create('RunAction')

EVT_POPUP_MENU, PopupMenuEvent = Events.create('PopupMenu')


class Panel(Window.PanelWithTools, Window.ThreadedPanelMixin):
    def __init__(self, parent, location, connection=None):
        self.location = location
        self.location.update_atime()
        self.plugins = []
        self.tools = []
        self.toolmap = {}
        self.startup_tool = None
        
        Window.PanelWithTools.__init__(self, parent)
        Window.ThreadedPanelMixin.__init__(self)
        
        EVT_POPUP_MENU(self, self.OpenPopupMenu)
        EVT_RUN_ACTION(self, self.OnRunAction)
        
        self.used_shortcuts = {}
        
        shortcuts = Config.getShortcuts()
        
        accel = []
        
        id = wx.NewId()
        self.Bind(wx.EVT_MENU, lambda e: self.HighlightUrl(), id=id)
        accel.append(Window.ParseShortcutAccel(shortcuts.get('focusurl', 'Ctrl+L'), id))
        
        self.SetAcceleratorTable(wx.AcceleratorTable(accel))
        
        wx.CallAfter(self.FocusUrl)
        wx.CallAfter(self.SetupConnection)
        
    def PostConnectionJob(self, *args, **kwargs):
        Connection.server.PostConnectionJob(Window.AsyncUpdateHandler(self), self.location, *args, **kwargs)

    def SetupConnection(self):
        def _(connection):
            with logger.begin('Checking plugin support', 'plugins.check', text=self.location.path, group=self.location.getLogGroup()) as msg:
                path = self.location.path
                if path is None or len(path) == 0: path = './'
                item = connection.lstat(path)
                connection.follow_link(posixpath.dirname(path), item)
                if self.location.path not in Model.model[self.location]:
                    Model.model.addFile(self.location, self.location.path, {'stat':item})
                return Plugins.GetSupported(self.location, connection)
        self.PostConnectionJob(method=_, on_complete=self.OnConnected, on_error=self.OnConnectionError)
        
    def OnConnected(self, plugins):
        self.plugins = plugins
        item = Model.model[self.location][self.location.path]
        if 'stat' in item and hasattr(item['stat'], 'linktarget'):
            self.location.path = item['stat'].linktarget
        self.SetupPlugins()
    
    def OnConnectionError(self, *args, **kwargs):
        self.Bookmarks(initial_url=self.location.getUrl())
        
        
    def CreatePanel(self, parent):
        self.SetBackground(Images.GetColor('toolbar', 'background'))
        self.url = UrlPanel.UrlPanel(self)
        self.throbber = self.url.throbber
        self.url.AddButton(Images.GetIcon('favourites', 'tall'), 'Bookmarks', self.Bookmarks)
        return self.CreateInnerControl(parent)

    def CreateInnerControl(self, parent):
        pass

    def Decorate(self, control):
        for scm in self.plugins:
            if hasattr(scm, 'decorate'):
                scm.decorate(control, self.location)

    def SetSizer(self, sizer):
        sizer.Add(self.tb, 0, wx.EXPAND)
        sizer.Add(self.url, 0, wx.EXPAND)
        sizer.Add(self.mainctrl, 1, wx.EXPAND|wx.ALL, 1)
        self.url.MoveBeforeInTabOrder(self.mainctrl)
        # self.urlpanel.SetSizer(self.urlpanel_sizer)
        wx.Panel.SetSizer(self, sizer)

        
    def SetupToolbar(self):
        self.tb.Add(lambda:1, 'dummy', ' ', Images.GetIcon('spacer'))


    def SetupPlugins(self):
        self.CreateToolbar()
        
        self.SetupPluginTools()
        
        if not self.location.isBookmarked():
            self.url.AddButton(Images.GetIcon('add_favourite', 'tall'), 'Add Bookmark', self.Bookmark)
        for scm in self.plugins:
            if hasattr(scm, 'get_views'):
                for tool in scm.get_views(self):
                    self.url.AddButton(Images.GetIcon(tool['icon'], 'tall'), tool['label'], self.curriedViewEvent(tool['class']))
        self.InitToolbarState()
        self.tb.DoLayout()
        self.GetSizer().Clear()
        vsizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(vsizer)
        vsizer.Layout()
        self.Layout()
        self.UpdateTools()
        
        self.PostEvent(Window.SetupCompleteEvent())
        if self.startup_tool:
            self.PostEvent(Window.RunToolEvent(tool=self.startup_tool))
            self.startup_tool = None
            
        
    def SetupPluginTools(self):
        self.tools = {}
        overrides = []
        for scm in self.plugins:
            self.tools[scm.shortName] = []
            for t in scm.get_tools(self, self.location):
                if t.override: overrides.append(t)
                self.tools[scm.shortName].append(t)
                self.toolmap[scm.shortName + '.' + t.name] = t
        for tool in overrides:
            scm, name = tool.override.split('.')
            for i in range(0, len(self.tools[scm])):
                if self.tools[scm][i].name == name:
                    # if not tool.default_shortcut and self.tools[scm][i].default_shortcut:
                    #     tool.default_shortcut = self.tools[scm][i].default_shortcut
                    self.tools[scm][i] = tool
                    
    
    def UpdateTools(self, connection=None):
        for scm in self.tools:
            for tool in self.tools[scm]:
                enable = tool.IsAble()
                if tool.can_toolbar:
                    self.tb.Enable(tool.name, enable)
                if hasattr(tool, 'menu_item'):
                    self.menus[scm].Enable(tool.menu_item.GetId(), enable)
    
    def UpdateMenu(self, mb):
        self.menubar = mb
        self.menus = {}
        self.menutools = {}
        shortcuts = Config.getShortcuts()
        insert_pos = mb.GetMenuCount() - 1
        for scm in self.plugins:
            scm = scm.shortName
            if scm in self.tools:
                menu = None
                for tool in self.tools[scm]:
                    if tool.can_menu:
                        if menu is None: menu = wx.Menu()
                        label = tool.label
                        sc = shortcuts.get(scm + '.' + tool.name, tool.default_shortcut)
                        if sc != '' and sc not in self.used_shortcuts:
                            self.used_shortcuts[sc] = tool
                            label += "\t" + sc
                        tool.menu_item = menu.Append(-1, label, tool.description)
                        self.menutools[tool.menu_item.GetId()] = tool
                        wx.EVT_MENU(self.GetGrandParent().GetParent(), tool.menu_item.GetId(), self.OnMenu)
                if menu is not None:
                    self.menubar.Insert(insert_pos, menu, [plugin for plugin in self.plugins if plugin.shortName == scm][0].label)
                    insert_pos += 1
                    self.menus[scm] = menu
                    
    def OpenPopupMenu(self, e=None):
        shortcuts = Config.getShortcuts()
        all_items = []
        for scm in self.plugins:
            scm = scm.shortName
            items = []
            for tool in self.tools[scm]:
                if tool.can_menu and tool.uses_selection and tool.IsAble():
                    items.append((scm, tool))
            if len(items):
                items.append('-')
                all_items.append(items)
        if len(all_items):
            all_items[-1].pop()
            menu = wx.Menu()
            self._popup_ids = {}
            for items in all_items:
                for item in items:
                    if type(item) in (str, unicode):
                        menu.AppendSeparator()
                    else:
                        scm, tool = item[0], item[1]
                        label = tool.label
                        sc = shortcuts.get(scm + '.' + tool.name, tool.default_shortcut)
                        if sc != '':
                            label += "\t" + sc
                        id = menu.Append(-1, label, tool.description).GetId()
                        self._popup_ids[id] = tool
            self.Bind(wx.EVT_MENU, self.OnMenuItemClick)
            self.PopupMenu(menu)
            self.Bind(wx.EVT_MENU, None)
            menu.Destroy()
            del self._popup_ids
            
    def OnMenuItemClick(self, e):
        tool = self._popup_ids[e.GetId()]
        self.RunTool(tool)
        
    def OnMenu(self, e):
        tool = self.menutools[e.GetId()]
        self.RunTool(tool)

    def OnAsyncComplete(self, e):
        Window.ThreadedPanelMixin.OnAsyncComplete(self, e)
        wx.CallAfter(self.UpdateTools)
        
    def OnRunAction(self, e):
        kwargs = e.kw
        if 'action' in kwargs: del kwargs['action']
        self.RunAction(e.action, *e.args, **kwargs)
        
    def RunTool(self, tool, *args, **kwargs):
        if isinstance(tool, basestring):
            tool = self.toolmap[tool]
        r = tool(*args, **kwargs)
        if isinstance(r, Model.Action):
            self.RunAction(r)
        return r
        
    def RunAction(self, action, *args, **kwargs):
        if isinstance(action, basestring):
            plugin, klass = action.split('.', 1)
            action = Plugins.CreateClass('%s.actions.%s' % (plugin, klass), self.location, *args, **kwargs)
        self.PostConnectionJob(method=action, on_complete=action.on_complete)
        
    def Bookmarks(self, e=None, *args, **kwargs):
        import Bookmarks
        self.PostEvent(Window.PanelChangeEvent(Bookmarks.Panel, *args, **kwargs))
        
    def Bookmark(self, e=None):
        import Bookmarks
        self.PostEvent(Window.PanelChangeEvent(Bookmarks.Editor, bookmark=self.location, nextclass=self.__class__))

    def GetConfigId(self):
        return super(Panel, self).GetConfigId()

    def GetUrl(self):
        return self.url.GetValue()

    def GetTitle(self):
        t = []
        if self.location['label']:
            t.append(self.location['label'])
        if not self.location.isBookmarked():
            p = os.path.basename(self.location.path.rstrip('/.'))
            if len(t) == 0 or p != t[0]:
                t.append(p)
        return ' - '.join(t)
        
    def UpdateUrl(self):
        self.url.SetValue(Connection.GetUrl(self.location))

    def FocusUrl(self):
        self.url.FocusUrl()

    def HighlightUrl(self):
        self.url.HighlightUrl()

    def IsValid(self):
        return True
        
    def curriedViewEvent(self, klass):
        def c(e=None): wx.PostEvent(self, Window.PanelChangeEvent(klass, location=self.location))
        return c

