
import wx
import wx.stc as stc

from lib import logger
from Config import Config, ConfigDict
import dispatcher

from . import Events, Window


class LogCtrl(stc.StyledTextCtrl):
    def __init__(self, parent, pos=wx.DefaultPosition, size=wx.DefaultSize, style=wx.SIMPLE_BORDER):
        stc.StyledTextCtrl.__init__(self, parent, wx.NewId(), pos, size, style)
        self.SetMarginWidth(1,0)
        self.SetViewWhiteSpace(False)
        # self.SetUseAntiAliasing(True)
        font = Config.getFont('fixed')
        self.StyleSetSpec(stc.STC_STYLE_DEFAULT, "face:%s,size:%d,bold,fore:#999999" % (font.GetFaceName(), font.GetPointSize()))
        self.StyleClearAll()
        self.styles = {
            logger.DEBUG: 1,
            logger.INFO: 2,
            logger.WARNING: 3,
            logger.ERROR: 4,
            logger.CRITICAL: 5,
        }
        self.StyleSetSpec(self.styles[logger.DEBUG], "fore:#999999")
        self.StyleSetSpec(self.styles[logger.INFO], "fore:#666666")
        self.StyleSetSpec(self.styles[logger.WARNING], "fore:#006600,back:#bbffbb")
        self.StyleSetSpec(self.styles[logger.ERROR], "fore:#660000,back:#ffcccc")
        self.StyleSetSpec(self.styles[logger.CRITICAL], "fore:#ffff00,back:#ff0000")
        dispatcher.connect(self.OnConfigChange, 'change', 'Config')
    def OnConfigChange(self, key=None, value=None):
        if key == 'fonts.fixed':
            font = Config.getNativeFont(value)
            self.StyleSetSpec(stc.STC_STYLE_DEFAULT, "face:%s,size:%d,bold,fore:#999999" % (font.GetFaceName(), font.GetPointSize()))
            self.StyleClearAll()
            self.StyleSetSpec(self.styles[logger.DEBUG], "fore:#999999")
            self.StyleSetSpec(self.styles[logger.INFO], "fore:#666666")
            self.StyleSetSpec(self.styles[logger.WARNING], "fore:#006600,back:#bbffbb")
            self.StyleSetSpec(self.styles[logger.ERROR], "fore:#660000,back:#ffcccc")
            self.StyleSetSpec(self.styles[logger.CRITICAL], "fore:#ffff00,back:#ff0000")
    def OnKey(self, e):
        if e.GetKeyCode() != 9:
            e.Skip()
    def write(self, text, level=logger.INFO):
        self.SetReadOnly(False)
        p = self.GetTextLength()
        self.AppendText(text)
        self.StartStyling(p, 0xff)
        self.SetStyling(len(text), self.styles[level])
        self.ScrollToLine(self.GetLineCount())
        self.SetReadOnly(True)


class Frame(Window.Frame):
    def __init__(self, parent):
        Window.Frame.__init__(self, parent, 'Application Log')
        
        self.logs = {}
        self.CreateStatusBar()
        
        self.nb = wx.Choicebook(self, -1)
        
        vs = wx.BoxSizer(wx.VERTICAL)
        vs.Add(self.nb, 1, wx.EXPAND|wx.ALL, 4)
        self.SetSizer(vs)
        
        menuBar = wx.MenuBar()
        menu = wx.Menu()
        if wx.Platform == '__WXMSW__': a = 'Ctrl-Tab'
        else: a = 'Ctrl-}'
        item = wx.MenuItem(menu, -1, 'Next Tab\t' + a, 'Next tab')
        wx.EVT_MENU(self, item.GetId(), self.NextTab)
        menu.AppendItem(item)
        if wx.Platform == '__WXMSW__': a = 'Shift-Ctrl-Tab'
        else: a = 'Ctrl-{'
        item = wx.MenuItem(menu, -1, 'Previous Tab\t' + a, 'Previous tab')
        wx.EVT_MENU(self, item.GetId(), self.PreviousTab)
        menu.AppendItem(item)
        item = wx.MenuItem(menu, -1, '&Close\tCtrl-W', 'Close this window')
        wx.EVT_MENU(self, item.GetId(), self.OnClose)
        menu.AppendItem(item)
        menuBar.Append(menu, '&File')
        self.SetMenuBar(menuBar)

        accel = []
        
        id = wx.NewId()
        self.Bind(wx.EVT_MENU, self.OnClose, id=id)
        accel.append(Window.ParseShortcutAccel('Ctrl-Alt-L', id))
        
        self.SetAcceleratorTable(wx.AcceleratorTable(accel))
        
        logger.observe(self)
        if self.settings.get('visible', False):
            wx.CallAfter(self.ShowLower)
            
    def ShowLower(self):
        self.Show()
        self.Lower()
        
    def NextTab(self, evt):
        self.nb.AdvanceSelection()

    def PreviousTab(self, evt):
        self.nb.AdvanceSelection(False)
        
    @Window.synchronized
    def notify_new_group(self, name, group):
        self.logs[name] = LogCtrl(self.nb)
        if name == 'default': title = 'Log'
        else: title = name
        self.nb.AddPage(self.logs[name], title)
        self.nb.ChangeSelection(self.nb.GetPageCount() - 1)
        
    @Window.synchronized
    def notify_write(self, group, text, level):
        self.logs[group].write(text + '\n', level)
        
    def GetDefaultConfig(self):
        return {'size':(600, 500)}
    
    def GetCurrentConfig(self):
        return ConfigDict(visible=self.IsShown())
        
    def OnClose(self, e=None):
        self.SaveState()
        self.Show(False)
        if e and hasattr(e, 'Veto'): e.Veto()

