import os, sys, time

import wx
import wx.aui

import AppInfo
from Config import Config, Invocation
import Images
import LogWindow
import Transfers
import Preferences
import Window
from lib import logger
import Location
import Bookmarks
from . import Update, Timer

class ChildFrame(wx.aui.AuiMDIChildFrame):
    def __init__(self, parent, copy=None, location=None, NewPanelEvent=None):
        wx.aui.AuiMDIChildFrame.__init__(self, parent, -1, title='Bookmarks', style=wx.aui.AUI_NB_SCROLL_BUTTONS|wx.aui.AUI_NB_TAB_MOVE|wx.aui.AUI_NB_CLOSE_ON_ALL_TABS)
        
        if NewPanelEvent:
            self.panel = NewPanelEvent.CreatePanel(self)
        else:
            if copy is not None and hasattr(copy.panel, 'GetUrl'):
                self.panel = Bookmarks.Panel(self, initial_url=copy.panel.GetUrl())
            else:
                if location:
                    module = location.importModule()
                    self.panel = module.Panel(self, location)
                else:
                    self.panel = Bookmarks.Panel(self)
        # self.UpdatePage()
        
        sizer = wx.BoxSizer()
        sizer.Add(self.panel, 1, wx.EXPAND)
        self.SetSizer(sizer)
        
        Window.EVT_TITLE_CHANGE(self, self.SetPageTitle)
        Window.EVT_PANEL_CHANGE(self, self.OnChangePanel)
        Window.EVT_SETUP_COMPLETE(self, self.OnSetupComplete)
        wx.CallAfter(self.Layout)
        if hasattr(self.panel, 'Activate'):
            wx.CallAfter(self.panel.Activate)
    
    def SetPageTitle(self, e):
        title = e.getTitle()
        self.SetTitle(title)

    def Activate(self):
        """Called by parent window from EVT_NOTEBOOK_PAGE_CHANGED"""
        if hasattr(self.panel, 'Activate'):
            self.panel.Activate()
            
    def OnChangePanel(self, e):
        if hasattr(e, 'WantNewTab') and e.WantNewTab():
            e.Skip()
        else:
            sizer = self.GetSizer()
            sizer.Detach(self.panel)
            self.panel.Close(True)
            wx.CallAfter(self.panel.Destroy)
            self.panel = e.CreatePanel(self)
            # try:
            #     self.panel = e.CreatePanel(self)
            # except Exception, e:
            #     self.panel = None
            #     wx.CallAfter(wx.MessageDialog(self, str(e), 'Error', wx.OK|wx.ICON_ERROR).ShowModal)
            if self.panel:
                # print self.panel.location.getId(), self.panel.location.path
                if hasattr(self.panel, 'location') and self.panel.location and self.panel.location.isBookmarked():
                    bm = Config.getBookmark(self.panel.location.id)
                    if bm and bm.get('active_module', '') != self.panel.__module__:
                        import Location
                        location = Location.Location(**bm)
                        location.label = bm.label
                        location['active_module'] = self.panel.__module__
                        location.saveBookmark()
            else:
                import Bookmarks
                self.panel = Bookmarks.Panel(self)
            # self.UpdatePage()
            sizer.Add(self.panel, 1, wx.EXPAND)
            self.Layout()
            if hasattr(self.panel, 'Activate'):
                self.panel.Activate()
                
    def Close(self, *args):
        if hasattr(self.panel, 'Close'):
            self.panel.Close(*args)
        return wx.aui.AuiMDIChildFrame.Close(self, *args)
    def SaveState(self, *args):
        if hasattr(self.panel, 'SaveState'):
            self.panel.SaveState(*args)
        
    def OnSetupComplete(self, e):
        self.UpdatePage()
    def UpdatePage(self):
        frame = self.GetGrandParent()
        mb = frame.MakeMenuBar()
        self.panel.UpdateMenu(mb)
        self.SetMenuBar(mb)
        self.panel.UpdateTools()
        if frame:
            nb = frame.GetNotebook()
            if hasattr(self.panel, 'ICON') and self.panel.ICON:
                nb.SetPageBitmap(nb.GetPageIndex(self), wx.Bitmap(Images.GetIcon(self.panel.ICON, 'small')))
            else:
                nb.SetPageBitmap(nb.GetPageIndex(self), wx.NullBitmap)



class Frame(wx.aui.AuiMDIParentFrame):
    _instancecount = 0
    _instances = {}
    _hidden_frame = None
    
    def __init__(self, parent, title=None, id=None, args=[]):
        self.args = args
        
        if title is None: title = AppInfo.title + ' v' + AppInfo.version
        
        Frame._instancecount += 1
        if id is None: id = 'frame-%d' % (Frame._instancecount,)
        self._instanceid = id
        Frame._instances[self._instanceid] = self
        
        style = wx.DEFAULT_FRAME_STYLE
        size = (760, 500)
        pos = (32, 32)
            
        wx.aui.AuiMDIParentFrame.__init__(self, parent, wx.ID_ANY, title, pos=pos, size=size, style=style)
        if id != '__hidden__':
            Config.RestoreWindow(self, self.GetConfigId())
        
        self.sb = self.CreateStatusBar(2)
        self.mainmenu = self.MakeMenuBar()
        self.SetMenuBar(self.mainmenu)
        
        wx.EVT_MENU(self, wx.ID_EXIT, self.OnClose)
        wx.EVT_CLOSE(self, self.OnClose)
        
        if wx.Platform == '__WXMSW__':
            ib=wx.IconBundle()
            ib.AddIconFromFile(Images.GetIcon("msw-ico"), wx.BITMAP_TYPE_ANY)
            self.SetIcons(ib)
        
        nb = self.GetNotebook()
        nb.Bind(wx.aui.EVT_AUINOTEBOOK_PAGE_CHANGED, self.OnPageChanged)
        
        wx.EVT_KEY_DOWN(nb, self.OnKey)

        Window.EVT_PANEL_CHANGE(self, self.OnChangePanel)
        
        Window.EVT_STATUS_PUSH(self, self.OnStatusPush)
        Window.EVT_STATUS_POP(self, self.OnStatusPop)
        Window.EVT_STATUS_TEXT(self, self.OnStatusText)
        
        wx.CallAfter(self.OpenArgs)
        wx.CallAfter(self.BindTabClick)
        
    def OpenArgs(self):
        c = 0
        for a in self.args:
            try:
                if a in Config.workspaces:
                    ws = Config.workspaces[a]
                    if len(ws.get('windows', [])) > 0:
                        del Frame._instances[self._instanceid]
                        self._instanceid = ws.windows[0].get('id', self._instanceid)
                        Frame._instances[self._instanceid] = self
                        c += self.open_workspace(ws.windows[0])
                        for ws_win in ws.windows[1:]:
                            f = self.Parent.NewWindow(id=ws_win.get('id', None))
                            c += f.open_workspace(ws_win)
                else:
                    bm = Config.findBookmark(a)
                    if bm:
                        location = Location.Location(**bm)
                    else:
                        location = Location.ParseUrl(a)
                    if location:
                        c += 1
                        child = ChildFrame(self, location=location)
                        child.Show()
                        if Invocation.options.tool:
                            child.panel.startup_tool = Invocation.options.tool
            except Exception, e:
                logger.exception(e)
                
        if c == 0:
            self.NewTab()
        
    def OnChangePanel(self, e):
        if hasattr(e, 'WantNewTab') and e.WantNewTab():
            self.NewTab(NewPanelEvent=e)
            
    def BindTabClick(self):
        tc = [x for x in self.GetNotebook().GetChildren() if type(x) is wx.aui.AuiTabCtrl]
        if len(tc):
            self.tabctrl = tc[0]
            wx.EVT_LEFT_DCLICK(self.tabctrl, self.OnDClick)
            
    def OnDClick(self, e):
        self.NewTab()
        
    def MakeMenuBar(self):
        self.menu_items = {}
        
        mb = wx.MenuBar()
        menu = wx.Menu()
        
        newItem = wx.MenuItem(menu, -1, 'New Window\tCtrl-N', 'Open a new window')
        wx.EVT_MENU(self, newItem.GetId(), self.Parent.NewWindow)
        menu.AppendItem(newItem)
        
        if self._instanceid is not '__hidden__':
            newItem = wx.MenuItem(menu, -1, 'New Tab\tCtrl-T', 'Open a new tab')
            wx.EVT_MENU(self, newItem.GetId(), self.NewTab)
            menu.AppendItem(newItem)
        
        menu.AppendSeparator()
        
        newItem = wx.MenuItem(menu, -1, 'Workspaces\tShift-Ctrl-W', 'Load and Edit saved workspaces')
        wx.EVT_MENU(self, newItem.GetId(), self.EditWorkspaces)
        menu.AppendItem(newItem)
        self.menu_editws = newItem
        self.menu_editws.Enable(len(Config.get('workspaces', {})) > 0)
        
        if self._instanceid is not '__hidden__':
            newItem = wx.MenuItem(menu, -1, 'Save Workspace\tShift-Ctrl-S', 'Save current workspace')
            wx.EVT_MENU(self, newItem.GetId(), self.SaveWorkspace)
            menu.AppendItem(newItem)
        
            menu.AppendSeparator()
        
            logItem = wx.MenuItem(menu, -1, 'Application Log\tCtrl-Alt-L', 'Show the application log window')
            wx.EVT_MENU(self, logItem.GetId(), self.ShowLogWindow)
            menu.AppendItem(logItem)
        
            twItem = wx.MenuItem(menu, -1, 'Transfers Window\tCtrl-Alt-T', 'Show the transfers window')
            wx.EVT_MENU(self, twItem.GetId(), self.ShowTransfersWindow)
            menu.AppendItem(twItem)
        
            menu.AppendSeparator()
            
            if wx.Platform == '__WXMSW__': a = 'Ctrl-Tab'
            else: a = 'Ctrl-}'
            item = wx.MenuItem(menu, -1, 'Next Tab\t' + a, 'Next tab')
            wx.EVT_MENU(self, item.GetId(), self.NextTab)
            menu.AppendItem(item)
            if wx.Platform == '__WXMSW__': a = 'Shift-Ctrl-Tab'
            else: a = 'Ctrl-{'
            item = wx.MenuItem(menu, -1, 'Previous Tab\t' + a, 'Previous tab')
            wx.EVT_MENU(self, item.GetId(), self.PreviousTab)
            menu.AppendItem(item)
        
            exitItem = wx.MenuItem(menu, -1, 'Close Tab\tCtrl-W', 'Close current tab')
            wx.EVT_MENU(self, exitItem.GetId(), self.CloseTab)
            menu.AppendItem(exitItem)
        
        if wx.Platform == '__WXMSW__': menu.AppendSeparator()
        prefsItem = wx.MenuItem(menu, -1, 'Preferences\tF7', 'Application preferences')
        wx.EVT_MENU(self, prefsItem.GetId(), self.Parent.OpenPrefs)
        menu.AppendItem(prefsItem)
        wx.App.SetMacPreferencesMenuItemId(prefsItem.GetId())
        
        if wx.Platform == '__WXMSW__': menu.AppendSeparator()
        if wx.Platform == '__WXMSW__': a = 'Alt-F4'
        else: a = 'Ctrl-Q'
        exitItem = wx.MenuItem(menu, wx.ID_EXIT, 'E&xit\t' + a, 'Quit the program')
        menu.AppendItem(exitItem)
        self.Bind(wx.EVT_MENU, self.Parent.OnFileExit, exitItem)
        wx.App.SetMacExitMenuItemId(exitItem.GetId())
        mb.Append(menu, '&File')
        
        
        if self._instanceid is not '__hidden__':
            menu = wx.Menu()
        
            a = 'Ctrl-C'
            item = wx.MenuItem(menu, -1, 'Copy\t' + a, 'Copy selection to clipboard')
            wx.EVT_MENU(self, item.GetId(), self.ClipboardCopy)
            menu.AppendItem(item)
            self.menu_items['copy'] = item
            a = 'Ctrl-V'
            item = wx.MenuItem(menu, -1, 'Paste\t' + a, 'Paste clipboard')
            wx.EVT_MENU(self, item.GetId(), self.ClipboardPaste)
            menu.AppendItem(item)
            self.menu_items['paste'] = item
            menu.AppendSeparator()
            a = 'Ctrl-A'
            item = wx.MenuItem(menu, -1, 'Select All\t' + a, 'Select All')
            wx.EVT_MENU(self, item.GetId(), self.SelectAll)
            menu.AppendItem(item)
            self.menu_items['select_all'] = item
            a = 'Ctrl-D'
            item = wx.MenuItem(menu, -1, 'Select None\t' + a, 'Select None')
            wx.EVT_MENU(self, item.GetId(), self.SelectNone)
            menu.AppendItem(item)
            self.menu_items['select_none'] = item
            a = 'Shift-Ctrl-I'
            item = wx.MenuItem(menu, -1, 'Invert Selection\t' + a, 'Invert Selection')
            wx.EVT_MENU(self, item.GetId(), self.SelectInverse)
            menu.AppendItem(item)
            self.menu_items['select_inverse'] = item
        
            mb.Append(menu, '&Edit')
        
            menu = wx.Menu()
            item = wx.MenuItem(menu, wx.ID_ABOUT, '&About', 'About')
            wx.EVT_MENU(self, wx.ID_ABOUT, self.Parent.OpenPrefs)
            menu.AppendItem(item)
            item = wx.MenuItem(menu, -1, '&Getting Started', 'Getting Started Documentation')
            wx.EVT_MENU(self, item.GetId(), self.ShowHelp)
            menu.AppendItem(item)
            mb.Append(menu, '&Help')
        
        return mb
        
    def UpdateMenu(self):
        return
        page = self.GetActiveChild()
        if page and hasattr(page, 'panel'):
            self.menu_items['copy'].Enable(hasattr(page.panel, 'ClipboardCopy'))
            self.menu_items['paste'].Enable(hasattr(page.panel, 'ClipboardPaste'))
            self.menu_items['select_all'].Enable(hasattr(page.panel, 'SelectAll'))
            self.menu_items['select_none'].Enable(hasattr(page.panel, 'SelectNone'))
            self.menu_items['select_inverse'].Enable(hasattr(page.panel, 'SelectInverse'))
        else:
            self.menu_items['copy'].Enable(False)
            self.menu_items['paste'].Enable(False)
            self.menu_items['select_all'].Enable(False)
            self.menu_items['select_none'].Enable(False)
            self.menu_items['select_inverse'].Enable(False)
            
    def ShowHelp(self, evt=None):
        import HelpPage
        nb = self.GetNotebook()
        for i in xrange(0, nb.GetPageCount()):
            if type(nb.GetPage(i)) is HelpPage.Panel:
                nb.SetSelection(i)
                return
        child = ChildFrame(self, NewPanelEvent=Window.PanelChangeEvent(HelpPage.Panel))
        child.Show()
        # page = HelpPage.Panel(nb)
        # nb.AddPage(page, page.GetTitle())
        # nb.SetSelection(nb.GetPageCount() - 1)
        
    def ClipboardCopy(self, evt):
        page = self.GetActiveChild()
        if (hasattr(page, 'panel') and hasattr(page.panel, 'ClipboardCopy')):
            if page.panel.ClipboardCopy(): return
        evt.Skip()
    def ClipboardPaste(self, evt):
        page = self.GetActiveChild()
        if (hasattr(page, 'panel') and hasattr(page.panel, 'ClipboardPaste')):
            if page.panel.ClipboardPaste(): return
        evt.Skip()
    def SelectAll(self, evt):
        page = self.GetActiveChild()
        if (hasattr(page, 'panel') and hasattr(page.panel, 'SelectAll')):
            if page.panel.SelectAll(): return
        evt.Skip()
    def SelectNone(self, evt):
        page = self.GetActiveChild()
        if (hasattr(page, 'panel') and hasattr(page.panel, 'SelectNone')):
            if page.panel.SelectNone(): return
        evt.Skip()
    def SelectInverse(self, evt):
        page = self.GetActiveChild()
        if (hasattr(page, 'panel') and hasattr(page.panel, 'SelectInverse')):
            if page.panel.SelectInverse(): return
        evt.Skip()

    def NextTab(self, evt):
        self.ActivateNext()
        
    def PreviousTab(self, evt):
        self.ActivatePrevious()
        
    def GetConfigId(self):
        return 'MainWindow-%s' % (str(self._instanceid),)
        
    def UpdateConfig(self):
        Config.SaveWindow(self, self.GetConfigId())
        
    def CloseTab(self, evt):
        page = self.GetActiveChild()
        if page:
            page.Close()
            page.Destroy()
        if 0 == self.GetNotebook().GetPageCount():
            self.OnClose()
            
    def OnClose(self, evt=None):
        try:
            nb = self.GetNotebook()
            for i in range(nb.GetPageCount()-1, -1, -1):
                page = nb.GetPage(i)
                page.Close()
            self.UpdateConfig()
            Config.save()
        except Exception, e:
            print str(e)
        self.Destroy()
        del Frame._instances[self._instanceid]
        if len(Frame._instances) == 0:
            if sys.platform == 'darwin':
                lw = AppInfo.topwindow.getLogWindow()
                if lw.IsShown():
                    lw.Show(False)
                # Frame._hidden_frame = self.Parent.NewWindow(id='__hidden__')
                # Frame._hidden_frame.Show(False)
            else:
                self.GetParent().Close()
            
    def OnPageChanged(self, e):
        page = self.GetActiveChild()
        if page: page.Activate()
        e.Skip()
        
    def OnKey(self, e):
        if e.GetKeyCode() == 9:
            self.GetActiveChild().panel.SetFocus()
        e.Skip()
        
    def NewTab(self, e=None, NewPanelEvent=None):
        copy = self.GetActiveChild()
        child = ChildFrame(self, copy, NewPanelEvent=NewPanelEvent)
        child.Show()
        wx.CallAfter(self.UpdateMenu)
        
    def OnStatusPush(self, e):
        self.GetStatusBar().PushStatusText(e.text)
        
    def OnStatusPop(self, e):
        self.GetStatusBar().PopStatusText()

    def OnStatusText(self, e):
        self.GetStatusBar().SetStatusText(e.text)

    def ShowLogWindow(self, e=None):
        lw = AppInfo.topwindow.getLogWindow()
        if lw.IsShown():
            lw.Show(False)
        else:
            lw.Show(True)
            lw.SetFocus()
        
    def ShowTransfersWindow(self, e=None):
        lw = AppInfo.topwindow.getTransfersWindow()
        if lw.IsShown():
            lw.Show(False)
        else:
            lw.Show(True)
            lw.SetFocus()

    def SaveState(self):
        nb = self.GetNotebook()
        for i in range(nb.GetPageCount()-1, -1, -1):
            page = nb.GetPage(i)
            page.SaveState()
        self.UpdateConfig()
        Config.save()
        
    def SaveWorkspace(self, e=None):
        from ui.base import Workspaces
        dlg = Workspaces.SaveDialog(self, Frame._instances.values())
        if dlg.ShowModal() == dlg.SAVE:
            self.menu_editws.Enable(True)
    
    def EditWorkspaces(self, e=None):
        frame = self.Parent.EditWorkspaces(e)
        if frame is not None:
            nb = self.GetNotebook()
            if nb.GetPageCount() == 1:
                page = nb.GetPage(0)
                if hasattr(page, 'panel') and isinstance(page.panel, Bookmarks.Panel):
                    self.Close()
        
    def get_workspace_description(self):
        pages = []
        nb = self.GetNotebook()
        for i in range(0, nb.GetPageCount()):
            page = nb.GetPage(i)
            pages.append(page.GetTitle())
        return ', '.join(pages)
        
    def get_workspace_data(self):
        window_data = {}
        pages = []
        nb = self.GetNotebook()
        for i in range(0, nb.GetPageCount()):
            page = nb.GetPage(i)
            if hasattr(page.panel, 'location') and page.panel.location:
                data = {}
                data['url'] = page.panel.location.getUrl()
                data['module'] = page.panel.__module__
                pages.append(data)
        if len(pages):
            del Frame._instances[self._instanceid]
            self._instanceid = "%s-%s" % (self._instanceid, str(time.time()))
            Frame._instances[self._instanceid] = self
            
            window_data['id'] = self._instanceid
            window_data['tabs'] = pages
            window_data['selection'] = nb.GetSelection()
        return window_data
    
    def open_workspace(self, ws):
        Config.RestoreWindow(self, self.GetConfigId())
        for tab in ws['tabs']:
            location = Location.ParseUrl(tab['url'])
            location['active_module'] = tab['module']
            child = ChildFrame(self, location=location)
            child.Show()
        self.GetNotebook().SetSelection(ws['selection'])
        return len(ws['tabs'])


class Top(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None)
        self.log = LogWindow.Frame(self)
        self.transfers = Transfers.Frame(self)
        AppInfo.transfer_queue = self.transfers
        if sys.platform == 'darwin':
            wx.MenuBar.MacSetCommonMenuBar(self.MakeMenuBar())
        frame = Frame(self, id='default', args=Invocation.args)
        frame.Show()
        self.frame = frame
        wx.CallLater(2000, self.AutoUpdate)
        
    def getLogWindow(self):
        return self.log
        
    def getTransfersWindow(self):
        return self.transfers

    def Close(self):
        self.log.Close()
        self.transfers.Close()
        wx.Frame.Close(self)
        
    def AutoUpdate(self):
        Update.check(self.frame)

    def MakeMenuBar(self):

        mb = wx.MenuBar()
        menu = wx.Menu()

        newItem = wx.MenuItem(menu, -1, 'New Window\tCtrl-N', 'Open a new window')
        wx.EVT_MENU(wx.GetApp(), newItem.GetId(), self.NewWindow)
        menu.AppendItem(newItem)

        newItem = wx.MenuItem(menu, -1, 'Workspaces\tShift-Ctrl-W', 'Load and Edit saved workspaces')
        wx.EVT_MENU(wx.GetApp(), newItem.GetId(), self.EditWorkspaces)
        menu.AppendItem(newItem)
        self.menu_editws = newItem
        self.menu_editws.Enable(len(Config.get('workspaces', {})) > 0)

        if wx.Platform == '__WXMSW__': menu.AppendSeparator()
        prefsItem = wx.MenuItem(menu, -1, 'Preferences\tF7', 'Application preferences')
        wx.EVT_MENU(wx.GetApp(), prefsItem.GetId(), self.OpenPrefs)
        menu.AppendItem(prefsItem)
        wx.App.SetMacPreferencesMenuItemId(prefsItem.GetId())

        if wx.Platform == '__WXMSW__': menu.AppendSeparator()
        if wx.Platform == '__WXMSW__': a = 'Alt-F4'
        else: a = 'Ctrl-Q'
        exitItem = wx.MenuItem(menu, wx.ID_EXIT, 'E&xit\t' + a, 'Quit the program')
        menu.AppendItem(exitItem)
        wx.EVT_MENU(wx.GetApp(), exitItem.GetId(), self.OnFileExit)
        wx.App.SetMacExitMenuItemId(exitItem.GetId())
        mb.Append(menu, '&File')

        return mb
    
    def NewWindow(self, e=None, id=None, args=[]):
        frame = Frame(self, id=id, args=args)
        frame.Show()
        return frame

    def EditWorkspaces(self, e=None):
        from ui.base import Workspaces
        dlg = Workspaces.Dialog(self)
        dlg.CenterOnScreen()
        r = dlg.ShowModal()
        if r == dlg.OPEN:
            ws = Config.workspaces.get(dlg.GetSelection(), None)
            if ws is not None:
                if len(ws.get('windows', [])) > 0:
                    return self.NewWindow(id=ws.windows[0].get('id', None), args=[dlg.GetSelection()])
        return None

    def OpenPrefs(self, evt):
        w = Preferences.Window(self)
        w.CenterOnScreen()
        w.ShowModal()

    def OnFileExit(self, e=None):
        for frame in Frame._instances.values():
            frame.Close()
        try:
            self.Close()
        except Exception, e:
            logger.exception(e)
