
import sys

import AppInfo

from Config import Config

class Base(object):
    def __init__(self, applicationName, notificationsList, appIconPath):
        pass
        
    def notify(self, noteType, title, text, icon=None, sticky=False, force=False):
        if Config.notifier.get('enabled', True) and (force or not AppInfo.is_active()):
            self.post_notification(noteType, title, text, icon, sticky)
            
    def post_notification(self, noteType, title, text, icon=None, sticky=False):
        pass


try:
    if sys.platform == 'darwin':
        import Growl
        
        class Notifier(Base):
            def __init__(self, applicationName, notificationsList, appIconPath):
                self.n = Growl.GrowlNotifier(applicationName=applicationName, notifications=notificationsList, applicationIcon=Growl.Image.imageFromPath(appIconPath))
                self.n.register()
            def post_notification(self, noteType, title, text, icon=None, sticky=False):
                if icon: icon = Growl.Image(icon)
                self.n.notify(noteType, title, text, icon=icon, sticky=sticky)

    else:
        import PySnarl
        if not PySnarl.snGetSnarlWindow():
            raise ImportError('Snarl not available')
        
        class Notifier(Base):
            def __init__(self, applicationName, notificationsList, appIconPath):
                self.applicationName = applicationName
                self.notificationsList = notificationsList
                self.appIconPath = appIconPath
            def post_notification(self, noteType, title, text, icon=None, sticky=False):
                if sticky: timeout = 0
                else: timeout = 2
                if not icon: icon = self.appIconPath
                PySnarl.snShowMessage(title, text, timeout, icon)

except ImportError:
    
    import wx
    import AppInfo
    import Images
    import ToasterBox
    
    from ui import Threading
    
    poller = Threading.Poller()
    
    class Notification(wx.MiniFrame):
        def __init__(self, noteType, title, text, icon=None, sticky=False):
            screen_w = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_X)
            screen_h = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_Y)
            wx.MiniFrame.__init__(self, None, 1, noteType, style=wx.DEFAULT_FRAME_STYLE|wx.TINY_CAPTION_HORIZ)
            self.SetBackgroundColour(wx.Colour(255, 255, 255))
            sizer = wx.BoxSizer(wx.VERTICAL)
            if noteType != title or not text:
                sizer.Add(wx.StaticText(self, -1, title), 0, wx.EXPAND|wx.ALL, 4)
            st = wx.StaticText(self, -1, text)
            st.SetBackgroundColour(wx.Colour(255, 255, 255))
            sizer.Add(st, 1, wx.EXPAND|wx.ALL, 4)
            self.SetSizer(sizer)
            pos = (16, 16)
            self.SetPosition(pos)
            self.SetMinSize((200, 100))
            sizer.Fit(self)
            self.Show()
            if not sticky:
                wx.CallLater(2000, self.Close)
            
    class Notifier(Base):
        def __init__(self, applicationName, notificationsList, appIconPath):
            self.applicationName = applicationName
            self.notificationsList = notificationsList
            self.appIconPath = appIconPath
        def post_notification(self, *args, **kwargs):
            poller.CallSynced(self.real_post_notification, *args, **kwargs)
        def real_post_notification(self, noteType, title, text, icon=None, sticky=False):
            style = ToasterBox.TB_CAPTION
            if title != noteType:
                if text == '': text = title
                else: text = title + "\n\n" + text
            elif text == '':
                text = title
                style = ToasterBox.DEFAULT_TB_STYLE
            tb = ToasterBox.ToasterBox(AppInfo.topwindow, ToasterBox.TB_SIMPLE, style, ToasterBox.TB_ONCLICK|ToasterBox.TB_ONTIME)
            tb.SetTitle(noteType)
            tb.SetPopupPauseTime(2500)
            tb.SetPopupScrollSpeed(1)
            
            screen_w = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_X)
            screen_h = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_Y)
            
            sizex = 200
            sizey = 124
            tb.SetPopupSize((sizex, sizey))

            posx = screen_w - (sizex + 8)
            posy = screen_h - (sizey + 8)
            tb.SetPopupPosition((posx, posy))
            tb.SetPopupBitmap(Images.GetImage('toaster'))
            
            tb.SetPopupBackgroundColor(wx.Colour(40, 98, 180))
            tb.SetPopupTextColor(wx.Colour(255, 255, 255))
            
            tb.SetPopupText(text)
            
            tb.SetPopupTextFont(wx.Font(10, wx.SWISS, wx.NORMAL, wx.NORMAL))
            
            tb.Play()
            # Notification(noteType, title, text)
