
from lib.platform import keychain

from . import dialog, controls

OK = dialog.OK
CANCEL = dialog.CANCEL

class Dialog(dialog.DialogBase):
    def __init__(self, location, message=''):
        self.location = location
        self.message = message
        dialog.DialogBase.__init__(self, None, location.getBaseUrl())
        if self.user.GetValue(): self.password.SetFocus()
        
    def CreatePanel(self):
        sizer = controls.sizers.Form()
        user = self.location.get('user', '')
        if user is None: user = ''
        self.user = controls.TextCtrl(self, user)
        sz = self.user.GetSize()
        self.user.SetMinSize((250, sz[1]))
        self.password = controls.PasswordCtrl(self)
        self.password.SetMinSize((250, sz[1]))
        self.store = controls.CheckBox(self, 'Store password in keychain')
        sizer.AddWidget(controls.Label(self, 'User Name:'), align=controls.sizers.RIGHT|controls.sizers.TOP|controls.sizers.BOTTOM, padding=4)
        sizer.AddWidget(self.user, padding=4)
        sizer.AddWidget(controls.Label(self, 'Password:'), align=controls.sizers.RIGHT|controls.sizers.TOP|controls.sizers.BOTTOM, padding=4)
        sizer.AddWidget(self.password, padding=4)
        sizer.AddWidget((0, 0), align=controls.sizers.RIGHT|controls.sizers.TOP|controls.sizers.BOTTOM, padding=4)
        sizer.AddWidget(self.store, padding=4)
        self.store.SetValue(self.settings.get('store_keychain', False))
        if self.message != '':
            vsizer = controls.sizers.VBox()
            vsizer.AddWidget(controls.Label(self, self.message), padding=8, align=controls.sizers.BOTTOM)
            vsizer.AddWidget(sizer)
            sizer = vsizer
        return sizer

    def GetUser(self):
        return self.user.GetValue()

    def GetPassword(self):
        return self.password.GetValue()

    def GetCurrentConfig(self):
        c = dialog.DialogBase.GetCurrentConfig(self)
        c['store_keychain'] = self.store.GetValue()
        return c

    def OnOK(self, e=None):
        self.location['user'] = self.GetUser()
        passwd = self.GetPassword()
        k = keychain.Keychain()
        if self.store.GetValue():
            k.setInternetPassword(self.location.get('scheme', 'ssh'), self.location['host'], self.location.get('port', 22), self.location.user, passwd)
            self.location['password'] = len(passwd) > 0
        else:
            k.setTemporaryPassword(self.location.getBaseUrl(), passwd)
        dialog.DialogBase.OnOK(self, e)
