import os

import wx

from Config import Config
import Connection
import Plugins

class Window(wx.Dialog):
    def __init__(self, parent):
        wx.Dialog.__init__(self, parent, wx.NewId(), 'Settings', size=(400, 300))
        
        self.nb = wx.Notebook(self)
        
        panel = wx.Panel(self.nb)
        
        self.stFont = wx.FontPickerCtrl(panel, -1, Config.getFont('fixed'))
        self.btnFileSelect = wx.Button(panel, -1, os.path.basename(Config.apps.get('editor', 'Choose')))
        self.chkNotifier = wx.CheckBox(panel, -1, 'Show Notifications')
        self.chkNotifier.SetValue(Config.notifier.get('enabled', True))
        auto_update = Config.auto_update.get('when', 1)
        choices = ['Never', 'When application starts', 'Weekly']
        auto_update = choices[auto_update]
        self.cbUpdates = wx.ComboBox(panel, -1, auto_update, choices=choices, style=wx.CB_DROPDOWN|wx.CB_READONLY)
        
        vsizer = wx.FlexGridSizer(0, 2, vgap=2, hgap=4)
        vsizer.SetFlexibleDirection(wx.VERTICAL)
        vsizer.AddGrowableCol(1)
        panel.SetSizer(vsizer)
        vsizer.Add(wx.StaticText(panel, -1, 'Monospace Font'), 0, wx.ALL, 4)
        vsizer.Add(self.stFont, 1, wx.ALL, 4)
        
        vsizer.Add(wx.StaticText(panel, -1, 'Editor'), 0, wx.ALL, 4)
        vsizer.Add(self.btnFileSelect, 1, wx.ALL, 4)
        
        vsizer.Add(wx.StaticText(panel, -1, 'Check for updates'), 0, wx.ALL, 4)
        vsizer.Add(self.cbUpdates, 1, wx.EXPAND|wx.ALL, 4)
        
        vsizer.Add(self.chkNotifier, 0, wx.ALL, 4)
        vsizer.Add((1, 1), 1, wx.EXPAND|wx.ALL, 4)
        
        self.nb.AddPage(panel, 'General')
        
        self.panels = []
        
        plugins = [getattr(Connection, C) for C in Connection.__all__ if hasattr(getattr(Connection, C), 'Preferences')]
        plugins += [getattr(Plugins, C) for C in Plugins.__all__ if hasattr(getattr(Plugins, C), 'Preferences')]
        for C in plugins:
            label = C.label
            name = C.shortName
            
            panel = wx.Panel(self.nb)
            
            p = C.Preferences(panel, Config.preferences.get(name, {}))

            self.panels.append(p)
            self.nb.AddPage(panel, label)
        
        main_sizer = wx.BoxSizer(wx.VERTICAL)
        main_sizer.Add(self.nb, 1, wx.EXPAND|wx.ALL, 8)
        self.SetSizer(main_sizer)
        
        self.SetMinSize((300, 100))
        
        wx.EVT_FONTPICKER_CHANGED(self, self.stFont.GetId(), self.OnFont)
        self.Bind(wx.EVT_BUTTON, self.OnSelectEditor, self.btnFileSelect)
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        wx.EVT_BUTTON(self, wx.ID_CANCEL, self.OnClose)
        
    def OnClose(self, e):
        Config.notifier.enabled = self.chkNotifier.GetValue()
        Config.auto_update.when = self.cbUpdates.GetSelection()
        for p in self.panels:
            if hasattr(p, 'OnClose'):
                p.OnClose()
        e.Skip()
        
    def OnSelectEditor(self, e=None):
        kwargs = dict(
            message="Choose an application",
            defaultFile="",
            style=wx.OPEN
        )
        if wx.Platform == '__WXMAC__':
            kwargs['defaultDir'] = '/Applications'
            kwargs['wildcard'] = '*.app'
        elif wx.Platform == '__WXMSW__':
            kwargs['wildcard'] = '*.exe'
        dlg = wx.FileDialog(self, **kwargs)
        
        # Show the dialog and retrieve the user response. If it is the OK response, 
        # process the data.
        if dlg.ShowModal() == wx.ID_OK:
            # This returns a Python list of files that were selected.
            paths = dlg.GetPaths()
            Config.apps['editor'] = paths[0]
            self.btnFileSelect.SetLabel(os.path.basename(paths[0]))
    def OnFont(self, event):
        font = self.stFont.GetSelectedFont()
        Config.setFont('fixed', font)
    
