import Queue

import wx

import lib.async
from . import Events

_ASYNC_EVENT = wx.NewEventType()

def EVT_ASYNC_UPDATE(win, func):
    win.Connect(-1, -1, _ASYNC_EVENT, func)

class AsyncUpdateEvent(Events.CommandEvent, lib.async.AsyncUpdateEventMixin):
    type = _ASYNC_EVENT

class Callback(object):
    def __init__(self, callback, *args, **kwargs):
        self.callback = callback
        self.args = args
        self.kwargs = kwargs
    def execute(self):
        self.callback(*self.args, **self.kwargs)

class Poller(wx.Timer):
    def __init__(self):
        self.q = Queue.Queue(0)
        wx.Timer.__init__(self)
        self.Start(50)
    def Notify(self):
        try:
            e = self.q.get(False)
        except Queue.Empty:
            pass
        else:
            try:
                e.execute()
                e.log()
            except lib.async.Repeat, x:
                self.q.put(AsyncUpdateEvent(job=e.job, src=e.src, status=''))
    def AsyncUpdate(self, e):
        self.q.put(AsyncUpdateEvent(**e))
    
    def CallSynced(self, callback, *args, **kwargs):
        self.q.put(Callback(callback, *args, **kwargs))
