import wx
import wx.lib.platebtn as platebtn

import Images
import Connection
import Location

import controls
import Window

platebtn.PlateButton.AcceptsFocusFromKeyboard = lambda self:False

class UrlPanel(Window.Panel):
    def __init__(self, parent):
        Window.Panel.__init__(self, parent, style=wx.TAB_TRAVERSAL)
        self.SetBackground(Images.GetColor('toolbar', 'background'))
        
        self.throbber = controls.Throbber(self, -1, Images.GetIcon('spinner'), size=(24, 24), frames=13, frameWidth=24, rest=0)
        self.throbber.UseBackgroundColour(False)
        
        self.url = controls.TextCtrl(self, parent.location.getUrl(), capture_enter=True)
        self.urlpanel_sizer = controls.sizers.HBox()
        self.urlpanel_sizer.AddWidget(self.url, 1, padding=4)
        self.urlpanel_sizer.AddWidget(self.throbber, 0, padding=4)
        
        self.initial = ''
        self.focussed = False
        self.url.Bind(wx.EVT_SET_FOCUS, self.OnUrlFocus)
        self.url.Bind(wx.EVT_KILL_FOCUS, self.OnUrlBlur)
        wx.EVT_KEY_UP(self.url, self.OnUrlKey)
        
        self.SetSizer(self.urlpanel_sizer)
        
    def AddButton(self, icon, text, on_click):
        btn = platebtn.PlateButton(self, -1, '', wx.Bitmap(icon), style=platebtn.PB_STYLE_GRADIENT)
        btn.SetPressColor(wx.Colour(*Images.GetColor('toolbar', 'pressed')))
        btn.SetToolTip(wx.ToolTip(text))
        self.urlpanel_sizer.AddWidget(btn, 0)
        wx.EVT_BUTTON(self, btn.GetId(), on_click)

    def GetValue(self):
        return self.url.GetValue()
    
    def SetValue(self, text):
        self.url.SetValue(text)
        
    def FocusUrl(self):
        self.url.SetFocus()
        self.url.SetInsertionPointEnd()

    def HighlightUrl(self):
        self.url.SetFocus()
        self.url.SelectAll()

    def OnUrlKey(self, e):
        if e.GetKeyCode() == 13:
            url = self.url.GetValue()
            self.PostEvent(Window.PanelChangeEvent(self.Parent.__class__, location=Location.ParseUrl(url)))
        elif e.GetKeyCode() == 27:
            self.url.SetValue(self.initial)
        e.Skip()

    def OnUrlFocus(self, e):
        self.focussed = True
        self.initial = self.url.GetValue()
        
    def OnUrlBlur(self, e):
        self.focussed = False

