
import wx
import wx.lib.buttons as buttons

class EditableListBox(wx.Panel):
    def __init__(self, parent, choices=[], entry_label=''):
        self.entry_label = entry_label
        
        wx.Panel.__init__(self, parent, -1)
        
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        bvsizer = wx.BoxSizer(wx.VERTICAL)
        
        self.list = wx.ListBox(self, -1, choices=choices, style=wx.LB_SORT|wx.LB_SINGLE)
        
        hsizer.Add(self.list, 2, wx.EXPAND|wx.ALL)
        
        for btn in self.CreateButtons(self):
            bvsizer.Add(btn, 0, wx.ALIGN_BOTTOM)
        
        hsizer.Add(bvsizer, 0, wx.ALIGN_BOTTOM|wx.LEFT, 4)
        
        self.SetSizer(hsizer)
        
        wx.EVT_LISTBOX(self, self.list.GetId(), self.OnSelectItem)
        wx.EVT_LISTBOX_DCLICK(self, self.list.GetId(), self.OnDoubleClick)
        
        self.UpdateState(False)
        
    def CreateButton(self, parent, label, OnClick):
        btn = buttons.ThemedGenButton(parent, -1, label)
        wx.EVT_BUTTON(self, btn.GetId(), OnClick)
        return btn
        
    def CreateButtons(self, parent):
        self.add = self.CreateButton(parent, 'Add', self.OnAdd)
        self.remove = self.CreateButton(parent, 'Remove', self.OnRemove)
        self.edit = self.CreateButton(parent, 'Edit', self.OnEdit)
        return [self.add, self.remove, self.edit]
        
    def UpdateState(self, have_selection):
        self.remove.Enable(have_selection)
        self.edit.Enable(have_selection)

    def OnSelectItem(self, e):
        self.UpdateState(self.list.GetSelection() != wx.NOT_FOUND)
    
    def OnDoubleClick(self, e):
        self.OnEdit(e)
        
    def OnAdd(self, e):
        d = wx.TextEntryDialog(self.Parent, self.entry_label)
        if wx.ID_OK == d.ShowModal():
            text = d.GetValue()
            self.list.Append(text)
            
    def OnRemove(self, e):
        self.RemoveSelected()
        
    def OnEdit(self, e):
        d = wx.TextEntryDialog(self.Parent, self.entry_label)
        d.SetValue(self.list.GetStringSelection())
        if wx.ID_OK == d.ShowModal():
            text = d.GetValue()
            self.RenameSelected(text)
    
    def RemoveSelected(self):
        self.list.Delete(self.list.GetSelection())
        self.UpdateState(self.list.GetSelection() != wx.NOT_FOUND)
        
    def RenameSelected(self, text):
        self.list.Delete(self.list.GetSelection())
        self.list.Append(text)
        self.UpdateState(self.list.GetSelection() != wx.NOT_FOUND)
    
    def GetStrings(self):
        return self.list.GetStrings()
    
    def GetSelection(self):
        return self.list.GetStringSelection()
