
import os
import wx
import wx.lib.buttonpanel as bp

from .. import Events

EVT_RUN_TOOL, RunToolEvent = Events.create('RunTool')

class Toolbar(bp.ButtonPanel):
    def __init__(self, parent, vertical=False, gradient=True, textalign='right', showlabels=True):
        self.showlabels = showlabels
        textalign = 'BP_BUTTONTEXT_ALIGN_' + textalign.upper()
        self.textalign = getattr(bp, textalign)
        if vertical: align = bp.BP_ALIGN_BOTTOM
        else: align = bp.BP_ALIGN_LEFT
        if gradient: style = bp.BP_USE_GRADIENT
        else: style = bp.BP_GRADIENT_NONE
        bp.ButtonPanel.__init__(self, parent, wx.NewId(), style=style, alignment=align)
        bpArt = self.GetBPArt()
        bpArt.SetColor(bp.BP_TEXT_COLOR, wx.Colour(50,50,50))
        bpArt.SetColor(bp.BP_BUTTONTEXT_COLOR, wx.Colour(50,50,50))
        bpArt.SetColor(bp.BP_GRADIENT_COLOR_TO, wx.Colour(196,196,196))
        bpArt.SetColor(bp.BP_GRADIENT_COLOR_FROM, wx.Colour(150,150,150))
        # bpArt.SetColor(bp.BP_GRADIENT_COLOR_TO, wx.Colour(196,196,196))
        # bpArt.SetColor(bp.BP_GRADIENT_COLOR_FROM, wx.Colour(170,170,170))
        bpArt.SetColor(bp.BP_SELECTION_BRUSH_COLOR, wx.Colour(110,110,110))
        bpArt.SetColor(bp.BP_SELECTION_PEN_COLOR, wx.Colour(110,110,110))
        bpArt.SetColor(bp.BP_BORDER_COLOR, wx.Colour(150,150,150))
        bpArt.SetMetric(bp.BP_BORDER_SIZE, 0)
        if False and gradient:
            self.SetTheme({
                'text':wx.WHITE,
                'border':wx.Colour(120,23,224),
                'gradient': {
                    'from': wx.Colour(60,11,112),
                    'to': wx.Colour(23,90,224)
                },
                'buttontext':wx.Colour(70,143,255),
                'separator':bp.BrightenColour(wx.Colour(60, 11, 112), 0.85),
                'selection': {
                    'brush':wx.Color(225, 225, 255),
                    'pen':wx.SystemSettings_GetColour(wx.SYS_COLOUR_ACTIVECAPTION)
                }
            })
        
        self.tools = {}
        self.names = {}
    
    def AcceptsFocus(self):
        return False
        
    def SetTheme(self, colours):
        return
        bpArt = self.GetBPArt()
        
        # set the color the text is drawn with
        bpArt.SetColor(bp.BP_TEXT_COLOR, colours['text'])

        bpArt.SetColor(bp.BP_BORDER_COLOR, colours['border'])
        bpArt.SetColor(bp.BP_GRADIENT_COLOR_FROM, colours['gradient']['from'])
        bpArt.SetColor(bp.BP_GRADIENT_COLOR_TO, colours['gradient']['to'])
        bpArt.SetColor(bp.BP_BUTTONTEXT_COLOR, colours['buttontext'])
        bpArt.SetColor(bp.BP_SEPARATOR_COLOR, colours['separator'])
        bpArt.SetColor(bp.BP_SELECTION_BRUSH_COLOR, colours['selection']['brush'])
        bpArt.SetColor(bp.BP_SELECTION_PEN_COLOR, colours['selection']['pen'])
        
    def Add(self, handler, name=None, label=None, icon=None, help=None):
        if name is None:
            name = handler.name
            label = handler.label
            icon = handler.icon
            help = handler.description
        if help is None: help = label
        if type(icon) == str:
            disabled = icon.replace('.png', '-disabled.png')
            if os.path.isfile(disabled): disabled = wx.Bitmap(disabled)
            else: disabled = None
            icon = wx.Bitmap(icon)
        else:
            disabled = None
        btn = bp.ButtonInfo(self, wx.NewId(), icon, kind=wx.ITEM_NORMAL, shortHelp=help)
        if disabled is not None:
            btn.SetBitmap(disabled, 'Disabled')
        if self.showlabels:
            btn.SetText(label)
            btn.SetTextAlignment(self.textalign)
        self.AddButton(btn)
        self.Bind(wx.EVT_BUTTON, self.OnClick, id=btn.GetId())
        self.tools[name] = {
            'handler': handler,
            'button': btn,
            'enabled': True
        }
        self.names[btn.GetId()] = name
    
    def Enable(self, name, enable=True):
        self.tools[name]['button'].Enable(enable)
        self.tools[name]['enabled'] = enable
        self.Refresh()
    def Enabled(self, name):
        return self.tools[name]['enable']
    def GetHandler(self, name):
        return self.tools[name]['handler']
    def OnClick(self, event):
        tool = self.tools[self.names[event.GetId()]]
        wx.PostEvent(self, RunToolEvent(tool=tool['handler']))
        # self.RunTool(tool)
    def RunTool(self, tool, *args, **kwargs):
        if type(tool) is str: tool = self.tools[tool]
        return tool['handler'](*args, **kwargs)

class DebugToolbar(Toolbar):
    def RunTool(self, tool, *args, **kwargs):
        try:
            super(DebugToolbar, self).RunTool(tool, *args, **kwargs)
        except Exception, e:
            import traceback, sys
            s = str(e)
            s += "\n" + "\n".join(traceback.format_tb(sys.exc_info()[2]))
            wx.MessageDialog(self, s, 'Error', wx.OK|wx.ICON_ERROR).ShowModal()


